### Not yet implemented.  Collecting ideas and code snippets until it can be built.

# 'phone_us': re.compile(r'((?:\D|^)([+]?1?)?[ -)(.]?(\d{3}[-\. ]??\d{3}[-\. ]??\d{4}|\(\d{3}\)\s*\d{3}[-\. ]??\d{4}|\d{3}[-\. ]??\d{4})(?:\D|$))'),
        

# https://github.com/daviddrysdale/python-phonenumbers


""" getTerminalSize()
 - get width and height of console
 - works on linux,os x,windows,cygwin(windows)
 
 http://code.activestate.com/recipes/440694-determine-size-of-console-window-on-windows/
"""

import platform, sys

try:
    from colorama import Fore, Back, Style, just_fix_windows_console
    colorize=True
except Exception as e:
    print('Colorama module failed to load.  Colorized output will be disabled.')
    colorize=False

current_os = platform.system()
if current_os == "Windows":
    just_fix_windows_console()

if colorize:
    WARN=Fore.YELLOW
    ERROR=Fore.RED
    NORMAL=Fore.RESET
else:
    WARN=''
    ERROR=''
    NORMAL=''

__all__=['getTerminalSize', 'warn', 'error', 'normal']


def getTerminalSize():
    '''
    Returns a tuple of (X,Y) screen dimensions
    '''
    tuple_xy=None
    if current_os == 'Windows':
        tuple_xy = _getTerminalSize_windows()
        if tuple_xy is None:
            tuple_xy = _getTerminalSize_tput()
            # needed for window's python in cygwin's xterm!
    if current_os == 'Linux' or current_os == 'Darwin' or  current_os.startswith('CYGWIN'):
        tuple_xy = _getTerminalSize_linux()
    if tuple_xy is None:
        #default
        tuple_xy = (80, 25)      # default value
    return tuple_xy

def _getTerminalSize_windows():
    res=None
    try:
        from ctypes import windll, create_string_buffer

        # stdin handle is -10
        # stdout handle is -11
        # stderr handle is -12

        h = windll.kernel32.GetStdHandle(-12)
        csbi = create_string_buffer(22)
        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
    except:
        return None
    if res:
        import struct
        (bufx, bufy, curx, cury, wattr,
         left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
        sizex = right - left + 1
        sizey = bottom - top + 1
        return sizex, sizey
    else:
        return None

def _getTerminalSize_tput():
    # get terminal width
    # src: http://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
    try:
        import subprocess
        proc=subprocess.Popen(["tput", "cols"],stdin=subprocess.PIPE,stdout=subprocess.PIPE)
        output=proc.communicate(input=None)
        cols=int(output[0])
        proc=subprocess.Popen(["tput", "lines"],stdin=subprocess.PIPE,stdout=subprocess.PIPE)
        output=proc.communicate(input=None)
        rows=int(output[0])
        return (cols,rows)
    except:
        return None


def _getTerminalSize_linux():
    def ioctl_GWINSZ(fd):
        try:
            import fcntl
            import termios
            import struct
            import os
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,'1234'))
        except:
            return None
        return cr
    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        try:
            cr = (env['LINES'], env['COLUMNS'])
        except:
            return None
    return int(cr[1]), int(cr[0])

def normal(s: str):
    print(NORMAL + s)

def status(s: str):
    sys.stdout.write('\r'+s)
    sys.stdout.flush()

def warn(s: str):
    print(WARN + '[warn ] ' + s + NORMAL)

def error(s: str):
    print(ERROR + '[error] ' + s + NORMAL)

if __name__ == "__main__":
    sizex,sizey=getTerminalSize()
    print('width =',sizex,'height =',sizey)
    error('This is error text')
    warn('This is warning text')
    normal('This is %s text' % ('normal'))


import multiprocessing as mp

import yaml

import piidigger.console as console

def processResult(outFilename: str,
                  queue: mp.Queue,
                  stopEvent: mp.Event,
                 ):
    
    
    try:
        with open(outFilename, 'w', encoding="utf-8") as of:
            while True:
                if stopEvent.is_set():
                    break
                result = queue.get()
                if result == None:
                    break
                yaml.dump(result, of, indent=4)
    except KeyboardInterrupt:
        pass
    except PermissionError as e:
        console.error(str(e))
        stopEvent.set()


'''Process DOCX files'''

import logging
import warnings
from logging.handlers import QueueHandler

from docx2python import docx2python

# Only need to modify the path during unit testing of this file.
if __name__=='__main__':
    import sys
    from pathlib import Path
    sys.path.insert(0, str(Path(__file__).absolute().parent.parent / ''))

import piidigger.globalfuncs as globalfuncs

warnings.filterwarnings('ignore', category=UserWarning, module='docx2python')

# Each filehandler must have the following:
#   "handles" -     dictionary to identify lists of file extensions and mime types that the handler will manage.
#                   This will be read by globals upon initial load to build the full list of supported mime types and file extensions
#   "processFile" - Function that manages opening and reading of the file.  The main module will call this handler wtih the "processFile(filename)" function.
#                   processFile should provide the lines of text to each of the dataHandlers

handles={
    'ext': [
        '.docx',
    ],
    'mime': [
        'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
        ],
}

def processFile(filename: str, dataHandlers: list, logConfig: dict) -> list:
    ''''
    Handle all file IO and text extraction operations for this file type.  Returns a list of results that have been validated by each datahandler.  
    "filename" is a string of the path and filename to process.  "handlers" is passed as a list of module objects that are called directly by processFile.
    '''

    logger=logging.getLogger('docx-handler')
    if not logger.handlers:
        logger.addHandler(QueueHandler(logConfig['q']))
    logger.setLevel(logConfig['level'])
    logger.propagate=False

    results={
        'filename': filename,
        'matches': dict()
    }

    try:
        # Read in all of the docx content and close the file
        docxContent=docx2python(filename)

        # Manipulate the text to get everything into one continuous string (space-separated)
        content=' '.join(docxContent.text.split('\n')).replace('\t', ' ') + ' ' + str(docxContent.core_properties)
        logger.debug('%s: Read %d bytes', filename, len(content))

        # Close the file now that we're done with it.
        docxContent.close()

        # Break up the text into bite-sized chunks for regex processing
        chunks=globalfuncs.makeChunks(content)
        logger.debug('%s: Created %d chunks', filename, len(chunks))

        del content

        for handler in dataHandlers:
            logger.debug('%s: Processing %d chunks with %s', filename, len(chunks), handler.dhName)
            for chunk in chunks:
                results=globalfuncs.processMatches(results, handler.findMatch(chunk), handler.dhName)
    except FileNotFoundError:
        logger.error('Previously discovered file no longer exists: %s. File skipped', filename)
    except PermissionError as e:
        logger.error('PermissionError adding %s.  File skipped.  Error message: %s', filename, str(e))
    except OSError as e:
        logger.error('OSError adding %s.  File skipped.  Error message: %s', filename, str(e))
    except Exception as e:
        logger.error('Unknown exception on file %s.  File skipped.  Error message: %s', filename, str(e))
        
    
    # Since Python sets aren't serializable as a JSON object type, we'll convert our results to Lists now. 
    logger.debug('%s: Rebuilding result sets into lists', filename)                   
    for handler in results['matches']:
        for key in results['matches'][handler]:
            l=list(results['matches'][handler][key])
            results['matches'][handler][key]=l

    logger.debug('%s: Processing complete', filename)
    return results


def main():
    from sys import argv
    from multiprocessing import Queue
    handlers=globalfuncs.getAllDataHandlerModules()
    logConfig={'q': Queue(),
                'level': "DEBUG",
                }
    for arg in argv[1:]:

        print(processFile(arg, handlers, logConfig))

if __name__ == '__main__':
    main()




import datetime
import logging
import os
import pathlib
import platform
import string

# try:
#     # If we're on Python later than 3.11 then tomllib is included.  We'll use the "toml" namespace so that the same code will work on older Pythons where toml had to be installed.
#     import tomllib as toml
# except ModuleNotFoundError:
import tomli

import piidigger.globalfuncs as globalfuncs
import piidigger.console as console
from piidigger.getmime import testMagic

logger=logging.getLogger(__name__)

class File:
    def __init__(self, f: pathlib.Path, mimeType: str):
        self.filename=f.name
        self.path=f.parent
        self.ext=f.suffix
        self.mimeType=mimeType
        self.handler=globalfuncs.getFileHandlerName(self.ext, self.mimeType)
        self.times=(f.stat().st_atime, f.stat().st_mtime)
        logger.debug('Initialized File object for %s, mimeType=%s, times=%s, handler=%s', self.getFullPath(), self.mimeType, str(self.times), self.handler)
        self.size=f.stat().st_size
        
    def __lt__(self, other):
        return self.getFullPath() < other.getFullPath()

    def getFullPath(self):
        return os.path.join(self.path, self.filename)
    
    def getExtension(self):
        return self.ext
    
    def getOldAccessTime(self):
        return self.times[0]
    
    def getOldModTime(self):
        return self.times[1]
    
    def getFileHandlerName(self) -> str:
        return self.handler
    
    
    def getFileSize(self):
        return self.size
    
    def getTimeStamps(self) -> tuple:
        return self.times

class Config:
    def __init__(self, configFile: str, useDefault: bool=False,):
        
        if useDefault:
            console.normal('Using default configuration.')
            self.config=globalfuncs.getDefaultConfig()
        else:
            try:
                with open(configFile, 'rb') as file:
                    self.config = tomli.load(file)
            except FileNotFoundError:
                console.warn('Configuration file %s not found. Using default configuration.' % configFile)
                configFile = 'Internal Config'
                self.config=globalfuncs.getDefaultConfig()
            except tomli.TOMLDecodeError as e:
                console.error('Invalid configuration (%s)' % (configFile))
                console.error(str(e))
                exit(globalfuncs.errorCodes['invalidConfig'])
        
        self.config['rootPath']=str(pathlib.Path(os.getcwd()).absolute())
        self.config['maxProcs']=os.cpu_count()
        hostname=str(platform.node())
        self.config['hostname']=hostname
        timeStamp=datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
        
        if _isAll(self.config['dataHandlers']):
            self.config['dataHandlers'] = globalfuncs.getSupportedDataHandlerNames()
        else:
            configHandlers=self.config['dataHandlers']
            notFound = [n for n in configHandlers if not n in globalfuncs.getSupportedDataHandlerNames()]
            if notFound:
                console.error("Unexpected data handler found in configuration file (%s)" % (configFile))
                console.error("The following data handlers will be ignored: " + str(notFound))
                self.config['dataHandlers'] = [n for n in configHandlers if n not in notFound]
        
        # Build the file names for each of the supported result file types
        outpath=self.config['results']['path']
        if not outpath.endswith('/'):
            outpath+='/'
        
        # We don't need this item now and it'll mess up the output file function if we leave it in.
        del self.config['results']['path']
        fileTypes=[('json', '.json'), 
                   ('text', '.txt')]

        for resultType, ext in fileTypes:
            filename=f"{outpath}{hostname}-{timeStamp}{ext}"
            # As set in the TOML file, the different results types are booleans.  We'll replace them with their pathname equiv if they were "enabled" to begin with.
            if self.config['results'][resultType]:
                self.config['results'][resultType]=filename
            else:
                del self.config['results'][type]

        # Fix any Windows paths replacing ALL with the actual drive letters and confirming that hard-coded starting directories exist
        if _isAll(self.config['includeFiles']['startDirs']['windows']):
            self.config['includeFiles']['startDirs']['windows'] = ['%s:\\' % d for d in string.ascii_uppercase if os.path.exists('%s:' % d)]        
        elif not all([os.path.isdir(d) for d in self.getStartDirs()]):
            console.error("Starting directory does not exist (%s). Check configuration file (%s)." % (self.getStartDirs(), configFile))
            exit(globalfuncs.errorCodes['invalidConfig'])


        # Replace "all" extensions with those currently supported by File Handlers
        if _isAll(self.config['includeFiles']['ext']):
            self.config['includeFiles']['ext'] = globalfuncs.getSupportedFileExts()
        else:
            # Before comparing to the expected list, fix the YAML-provided extensions if they don't start with a period
            configExts=[c if c.startswith('.') else '.' + c for c in self.config['includeFiles']['ext']]
            notFound = [n for n in configExts if not n in globalfuncs.getSupportedFileExts()]
            if notFound:
                console.error("Unexpected file extensions found in configuration file (%s)" % (configFile))
                console.error("The following file extensions will be ignored: " + str(notFound))
                self.config['includeFiles']['ext'] = [n for n in configExts if n not in notFound]
        
        # Replace "all" MIME types with those currently supported by File Handlers
        if testMagic():
            if _isAll(self.config['includeFiles']['mime']):
                self.config['includeFiles']['mime'] = globalfuncs.getSupportedFileMimes()
            else:
                configMimes=self.config['includeFiles']['mime']
                notFound = [n for n in configMimes if not n in globalfuncs.getSupportedFileMimes()]
                if notFound:
                    console.error("Unexpected MIME types found in configuration file (%s)" % (configFile))
                    console.error("The following MIME types will be ignored: " + str(notFound))
                    self.config['includeFiles']['mime'] = [n for n in configMimes if n not in notFound]
        else:
            self.config['includeFiles']['mime']=[]

        # Add Results and Log folders to the list of folders to exlude
        self.config['excludeDirs'][globalfuncs.getOSType()].append(str(pathlib.Path(self.getRootPath()) / outpath))
        self.config['excludeDirs'][globalfuncs.getOSType()].append(str(pathlib.Path(pathlib.Path(self.getRootPath()) / self.getLogFile()).parent))
    

    def getDataHandlers(self):
        return self.config['dataHandlers']
    
    
    def getEnabledOutputTypes(self):
        #return [key for key in self.config['results'].keys()]
        return self.config['results'].keys()
    

    def getExcludeDirs(self):
        return self.config['excludeDirs'][globalfuncs.getOSType()]
    

    def getFileExts(self):
        return self.config['includeFiles']['ext']
    

    def getConfig(self):
        return self.config
    

    def getLocalFilesOnly(self):
        return self.config['localFilesOnly']
    
    
    def getLogLevel(self):
        return self.config['logging']['logLevel']
    

    def getLogFile(self):
        return self.config['logging']['logFile']


    def getMaxProcs(self):
        return self.config['maxProcs']
    

    def getMimeTypes(self):
        return self.config['includeFiles']['mime']
    

    def getOutputFile(self, resultType: str = ""):
        if resultType:
            return self.config['results'][resultType]
        else:
            return self.config['results']
    

    def getRootPath(self):
        return self.config['rootPath']
    

    def getStartDirs(self):
        return self.config['includeFiles']['startDirs'][globalfuncs.getOSType()]
    
    
    def setMaxProcs(self, procs):
        self.config['maxProcs']=procs
        

def _isAll(x) -> bool:
    '''
    Checks if the provided content is equal to the string 'all'.  If it's a list or a dictionary, it will compare only the first item.

    Returns None if it's one of those three types.
    '''

    s=''
    if isinstance(x, list):
        s = x[0]
    if isinstance(x, str):
        s= x
    if isinstance(x, dict):
        s = list(x.keys())[0]
    
    return s.lower() == 'all'


if __name__=="__main__":
    config=Config('piidigger.toml')

    print(config.getConfig())


import codecs, logging
from logging.handlers import QueueHandler

if __name__=='__main__':
    import sys
    from pathlib import Path
    sys.path.insert(0, str(Path(__file__).absolute().parent.parent / ''))

import piidigger.globalfuncs as globalfuncs
from piidigger.getencoding import getEncoding

# Each filehandler must have the following:
#   "handles" -     dictionary to identify lists of file extensions and mime types that the handler will manage.
#                   This will be read by globals upon initial load to build the full list of supported mime types and file extensions
#   "processFile" - Function that manages opening and reading of the file.  The main module will call this handler wtih the "processFile(filename)" function.
#                   processFile should provide the lines of text to each of the dataHandlers

handles={
    'ext': [
        '.aplt', '.applescript', '.armx', '.asp', '.asax', '.asmx', '.aspx',
        '.bat',
        '.c', '.cc', '.cfm', '.clj', '.cljs', '.clojure', '.cob', '.cpp', '.csh', '.csv'
        '.erl',
        '.h', '.hrl', '.htm', '.ht4', '.html', '.html5',
        '.go', '.gvy',
        '.j', '.json', '.js', '.jsp',
        '.log',
        '.perl', '.php', '.pl', '.ps1', '.py',
        '.rb',
        '.scpt', '.sdef', '.ser', '.sh',
        '.txt',
        '.vb',
        '.xml',
        '.yaml',
        '.zsh',
    ],
    'mime': [
        'application/json', 
        'application/xml',
        'text/html', 
        'text/plain', 
    ],
}


def processFile(filename: str, 
                dataHandlers: list, 
                logConfig: dict,
               ) -> list:
    ''''
    Handle all file IO and text extraction operations for this file type.  Returns a list of results that have been validated by each datahandler.  
    "filename" is a string of the path and filename to process.  "handlers" is passed as a list of module objects that are called directly by processFile.
    '''

    logger=logging.getLogger('plaintext-handler')
    if not logger.handlers:
        logger.addHandler(QueueHandler(logConfig['q']))
    logger.setLevel(logConfig['level'])
    logger.propagate=False
    
    results={
        'filename': filename,
        'matches': {}
    }
    
    enc = getEncoding(filename)

    if enc == None:
        logger.info('Unknown encoding type (%s)', filename)
        results['matches']={}
        return results
    
    # After making our best guess with the encoding, replace any unexpected characters with a plain ASCII "?"
    # The risk is we could lose something important, but if that's the one piece of data anywhere on the file system that would have matched,
    # then it's a risk worth taking for a more stable discovery tool.  More likely is that we might miss ONE INSTANCE of data in a file system that has 
    # many more instances for discovery.

    # First we open the file, then we pass each line of text to the datahandler.  File IO is the bottle neck, so by reading each file just once, we should
    # be able to maintain reasonable performance.

    # Then we combine all the file content into one string before breaking it up into bite-sized chunks for regex processing
    try:
        with codecs.open(filename, 'r', encoding=enc, errors='replace') as f:
            lines=f.readlines()
            logger.debug('%s: Read %d lines', filename, len(lines))
            
            lines=[line.strip() for line in lines]
            
            content=' '.join(lines)
            logger.debug('%s: Joined %d lines into content string (len=%d)', filename, len(lines), len(content))

            # To conserve memory, especially on large files.  We only need the file content in one long string.
            del lines

            chunks=globalfuncs.makeChunks(content)
            logger.debug('%s: Created %d chunks', filename, len(chunks))

            del content

            for handler in dataHandlers:
                logger.debug('%s: Processing %d chunks with %s', filename, len(chunks), handler.dhName)
                for chunk in chunks:
                    results=globalfuncs.processMatches(results, handler.findMatch(chunk), handler.dhName)
                logger.debug('%s: %d chunks processed with %s', filename, len(chunks), handler.dhName)
    except FileNotFoundError:
        logger.error('Previously discovered file no longer exists: %s. File skipped', f.absolute())
    except PermissionError as e:
        logger.error('PermissionError adding %s.  File skipped.  Error message: %s', f.absolute(), str(e))
    except OSError as e:
        logger.error('OSError adding %s.  File skipped.  Error message: %s', f.absolute(), str(e))
    except UnicodeDecodeError as e:
        logger.error('Unicode error processing file %s (enc=%s): %s', filename, enc, e)
    except LookupError as e:
        logger.error('Codec lookup error processing file %s (enc=%s)', filename, enc)
    except Exception as e:
        logger.error('Unknown exception on file %s.  File skipped.  Error message: %s', filename, str(e))
        
    # Since Python sets aren't serializable as a JSON object type, we'll convert our results to Lists now.
    logger.debug('%s: Rebuilding result sets into lists', filename)
    for handler in results['matches']:
        for key in results['matches'][handler]:
            l=list(results['matches'][handler][key])
            results['matches'][handler][key]=l

    logger.debug('%s: Processing complete', filename)
    return results


def main():
    from sys import argv
    from multiprocessing import Queue
    handlers=globalfuncs.getAllDataHandlerModules()
    logConfig={'q': Queue(),
                'level': "DEBUG",
                }
    
    for arg in argv[1:]:

        print(processFile(arg, handlers, logConfig))

if __name__ == '__main__':
    main()




import pytest

from src.piidigger.globalfuncs import makeChunks

#The first test breaks up the string on a word boundary
#The second test is just a bit too long (next word boundary exceeds the chunk size), but still passes
#The third test is short enough that only one chunk is produced.
#The fourth test uses a single, too-long string that exceeds the chunk size, so the string is broken right at the chunksize.

@pytest.mark.parametrize('content, expected_result', [
                                ('Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Vitae proin sagittis nisl rhoncus mattis rhoncus urna neque. Neque viverra justo nec ultrices dui sapien. Turpis cursus in hac habitasse. Eu facilisis sed odio morbi quis. Commodo ullamcorper a lacus vestibulum. In metus vulputate eu scelerisque felis imperdiet. Orci a scelerisque purus semper eget duis at. Tincidunt augue interdum velit euismod. Dignissim enim sit amet venenatis urna cursus eget. Consequat mauris nunc congue nisi vitae suscipit tellus. Cursus euismod quis viverra nibh. Platea dictumst vestibulum rhoncus est pellentesque elit ullamcorper dignissim.', ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Vitae proin sagittis nisl rhoncus mattis rhoncus urna neque. Neque viverra justo nec ultrices dui sapien. Turpis cursus in hac habitasse. Eu facilisis sed odio morbi quis. Commodo ullamcorper a lacus vestibulum. In metus vulputate eu scelerisque felis imperdiet. Orci a scelerisque purus semper eget duis at. Tincidunt augue interdum velit euismod. Dignissim enim sit amet venenatis urna cursus eget. Consequat mauris nunc congue nisi vitae suscipit tellus. Cursus euismod quis viverra nibh. Platea dictumst vestibulum rhoncus', 'est pellentesque elit ullamcorper dignissim.',]),
                                ('Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Consectetur libero id faucibus nisl tincidunt eget nullam. Sem integer vitae justo eget magna fermentum. Elit duis tristique sollicitudin nibh sit amet commodo. Ullamcorper eget nulla facilisi etiam dignissim diam quis enim lobortis. Aliquam etiam erat velit scelerisque in dictum non. Posuere ac ut consequat semper viverra nam libero. Egestas diam in arcu cursus euismod. Gravida quis blandit turpis cursus in hac habitasse platea. Vestibulum rhoncus est pellentesque elit ullamcorper. Id eu nisl nunc mi ipsum faucibus. Nullam non nisi est sit amet facilisis magna. Posuere urna nec tincidunt praesent semper feugiat nibh sed. Consequat semper viverra nam libero. Tincidunt eget nullam non nisi est sit amet facilisis.', ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Consectetur libero id faucibus nisl tincidunt eget nullam. Sem integer vitae justo eget magna fermentum. Elit duis tristique sollicitudin nibh sit amet commodo. Ullamcorper eget nulla facilisi etiam dignissim diam quis enim lobortis. Aliquam etiam erat velit scelerisque in dictum non. Posuere ac ut consequat semper viverra nam libero. Egestas diam in arcu cursus euismod. Gravida quis blandit turpis cursus in hac habitasse platea. Vestibulum rhoncus est pellentesque elit ullamcorper. Id eu nisl nunc mi ipsum faucibus. Nullam', 'non nisi est sit amet facilisis magna. Posuere urna nec tincidunt praesent semper feugiat nibh sed. Consequat semper viverra nam libero. Tincidunt eget nullam non nisi est sit amet facilisis.',]),
                                ('This is a short test!', ['This is a short test!',]),
                                ('DK3dbQaQwKLPSSAS5QA785SyFpMd7f45thNSbHpbGBgy65diHfiG2yWnNjZn36wBquGw97wYGgR7RPXJWVCHiu84JmewHAr9NMZe0j2i2ycJCqPLKBikQ9yHxvAXEJXLfiRd7G9Hy5BA4LiyhZ1FLVtuyUg0TmCchX9g9GXqcNQN3nAnnV1zUAPY33YNU8CGNdrEEe0UifJy2yLLxS4zU91WrZWhB0tDZXtGWdYD0xuxKWaEpcjLa1FwtDQ2MWijRz5BQvFWveRFPhzeNchkuHHJLnpCtTSVzujDjC7GK1JKF3SZr31mrTjwdxkLCHbGjfqjHfFm4VHi6hZ9R9HLhGccbbSNCpbWmRj2d7Xed5rmh0v5xTZyQ1J7Mk1W7haFihwdJgj15XXf3yEMUnJmuaVuVywXKL9zNGKTEazcFCzit0Z01qFfN4GQc0mQ0MvR3qg1SPMJjGWUvfaT9CdXPffJ5j92xrwZTN4x53nBbHtdcNe1ekGfudUSaRf9ew8DVdUwufvYrpgLjg8pWCzXUayS7iyMS8KQmRYtEferaPGRScVcH1UrbNkQjSUeRLNxFZuVAdF21iNXwG57WQMTPZpuibKfAhHh3RFug7yjpNE8WDiL7N6GYwMBiMFRfS73RZcSf3HYAifqVYTyyc5nmRxFdNhJd456S1zxCwGGDDA9h44xqphTZfkz2j2Z', ['DK3dbQaQwKLPSSAS5QA785SyFpMd7f45thNSbHpbGBgy65diHfiG2yWnNjZn36wBquGw97wYGgR7RPXJWVCHiu84JmewHAr9NMZe0j2i2ycJCqPLKBikQ9yHxvAXEJXLfiRd7G9Hy5BA4LiyhZ1FLVtuyUg0TmCchX9g9GXqcNQN3nAnnV1zUAPY33YNU8CGNdrEEe0UifJy2yLLxS4zU91WrZWhB0tDZXtGWdYD0xuxKWaEpcjLa1FwtDQ2MWijRz5BQvFWveRFPhzeNchkuHHJLnpCtTSVzujDjC7GK1JKF3SZr31mrTjwdxkLCHbGjfqjHfFm4VHi6hZ9R9HLhGccbbSNCpbWmRj2d7Xed5rmh0v5xTZyQ1J7Mk1W7haFihwdJgj15XXf3yEMUnJmuaVuVywXKL9zNGKTEazcFCzit0Z01qFfN4GQc0mQ0MvR3qg1SPMJjGWUvfaT9CdXPffJ5j92xrwZTN4x53nBbHtdcNe1ekGfudUSaRf9ew8DVdUwufvYrpgLjg8pWCzXUayS7iyMS8KQmRYtEferaPGRScVcH1UrbNkQjSUeRLNxFZuVAdF21iNXwG57WQMTPZpuibKfAhHh3RFug7yjpNE8WDiL7N6GYwMBiMFRfS73RZcSf3HYAi', 'fqVYTyyc5nmRxFdNhJd456S1zxCwGGDDA9h44xqphTZfkz2j2Z'])
                            ]
                        )

def test_make_chunks(content, expected_result):
    result=makeChunks(content)

    assert result==expected_result


import os

import pytest

from src.piidigger.globalfuncs import writeDefaultConfig, getDefaultConfig
import tomli

def test_write_default_config():
    expectedConfig=getDefaultConfig()
    testFile = 'testDefault.toml'

    result=writeDefaultConfig(testFile)

    if result=="Success":
        with open(testFile, 'rb') as f:
            savedConfig=tomli.load(f)
    else:
        savedConfig={}

    os.remove(testFile)

    assert savedConfig==expectedConfig


# Not yet implemented.  Collecting ideas for eventual coding.

# Format of track data: https://www.magtek.com/content/documentationfiles/d99800004.pdf

# Maybe a working regex? https://stackoverflow.com/questions/2540865/regular-expression-for-credit-card-track-data 


import argparse
import logging
import multiprocessing as mp
import sys
import textwrap
import traceback
from ctypes import c_int, c_uint64
from datetime import datetime
from logging.handlers import QueueHandler
from os import makedirs, cpu_count
from pathlib import Path
from time import sleep


import json
from wakepy import keep


import piidigger.classes as classes
import piidigger.console as console
import piidigger.filescan as filescan
import piidigger.globalfuncs as globalfuncs


def cleanup(processes: dict,
            queues: dict,
            stopEvent: mp.Event,
            abort: bool=False,
           ):
    '''Cleans up all processes on ending the program'''

    #Set the stopEvent so that processes have a chance to shutdown normally
    stopEvent.set()

    if abort:
        sleep(.5)
        for processType in processes:
            for process in processes[processType]:
                if process.is_alive():
                    console.normal('Terminating process: %s' % process.name)
                    process.terminate()

    try:
        processes['progressLine'][0].join()
    except KeyboardInterrupt:
        pass

    # Clean up any queues that still have stuff in them.  This will ensure that the mp.Queue-based QueueHandler thread will also terminate
    print()
    for q in queues:
        if not queues[q].empty():
            console.normal('Cleaning up queue: %s' % q)
            globalfuncs.clearQ(queues[q])

def commandLineParser() -> argparse.ArgumentParser:
    '''Handles the command line options'''
    parser = argparse.ArgumentParser(
    prog=sys.argv[0].split('/')[-1],
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''
        Search the file system for Personally Identifiable Information

        NOTES:
            * All program configuration is kept in \'piidigger.toml\' -- a TOML-formatted configuration file
            * A default configuration will be used if the default 'piidigger.toml' file doesn't exist
        ''')
    )

    configControl = parser.add_argument_group(title='Configuration')
    configControl.add_argument(
        '-c', '--create-conf',
        dest='createConfigFile',
        default='',
        help='Create a default configuration file for editing/reuse.',
        )
    configControl.add_argument(
        '-d', '--default-conf',
        dest='defaultConfig',
        action='store_true',
        help='Use the default, internal config.'
        )
    configControl.add_argument(
        '-f', '--conf-file', 
        dest='configFile', 
        default='piidigger.toml', 
        help='path/to/configfile.toml configuration file (Default = "piidigger.toml").  If the file is not found, the default, internal configuration will be used.'
        )
    configControl.add_argument(
        '-p', '--max-process',
        dest='maxProc',
        default=0,
        type=int,
        help='Override the number processes to use for searching files.  Will use the lesser of CPU cores or this value.  On production servers, consider setting this to less than the number of physical CPUs.  See \'--cpu-count\' below.',
        )
    
    miscInfoControl = parser.add_argument_group(title='Misc. Info')
    miscInfoControl.add_argument(
        '--cpu-count',
        dest='cpuCount',
        action='store_true',
        help='Show the number of logical CPUs provided by the OS.  Use this to tune performance.  See \'--max-process\' above.'
    )
    miscInfoControl.add_argument(
        '--list-datahandlers',
        dest='listDH',
        action='store_true',
        help='Display the list of data handlers and exit'
    )
    miscInfoControl.add_argument(
        '--list-filetypes',
        dest='listFT',
        action='store_true',
        help='Display the list of file types and exit'
    )
    
    return parser.parse_args()


def logProcessor(config: classes.Config, 
                 queue: mp.Queue,
                 stopEvent: mp.Event):
    try:
        logger=logging.getLogger()

        logFileFormatter=logging.Formatter('%(asctime)s:[%(name)s]:%(levelname)s:%(message)s')
        logFileHandler=logging.FileHandler(filename=config.getLogFile(),mode='w',encoding='utf-8')
        logFileHandler.setFormatter(logFileFormatter)

        logger.setLevel(config.getLogLevel())
        logger.addHandler(logFileHandler)

        logger.info('Starting logProcessor (%s)', mp.current_process().pid)
        stopCause=None

        while True:
            if stopEvent.is_set():
                stopCause = 'stopEvent'
                break

            message = queue.get(1)

            if message == None:
                stopCause = 'endQueue'
                break

            logger.handle(message)
    except KeyboardInterrupt:
        # Give other processes a chance to write their final messages to the queue
        sleep(5)
        globalfuncs.clearQ(queue)
        stopCause='ctrlc'
        logger.info('[logProcessor]User terminated scan')
    finally:
        logger.info('[logProcessor]Stopping logProcessor (%s)', str(stopCause))
        

def fileHandlerDispatcher(config: classes.Config,
                          queues: dict,
                          totals: dict,
                          stopEvent: mp.Event,
                          activeFilesQProcesses: mp.Value,
                         ):

    try:
        with activeFilesQProcesses.get_lock():
            activeFilesQProcesses.value+=1
        dataHandlersModules=globalfuncs.getEnabledDataHandlerModules(config.getDataHandlers())

        logger=logging.getLogger('fileHandlerDispatch')
        if not logger.handlers:
            logger.addHandler(QueueHandler(queues['logQ']))
        logger.setLevel(config.getLogLevel())
        logger.propagate=False
        logger.debug('Process %s (%s) started (Active=%d)', mp.current_process().name, mp.current_process().pid, activeFilesQProcesses.value)
        
        logConfig={'q': queues['logQ'],
                'level': config.getLogLevel(),
                }

        while True:
            if stopEvent.is_set():
                break
            item=queues['filesQ'].get()
            if item == None:
                queues['filesQ'].put(item)
                break
            filename=item.getFullPath()
            fileHandlerModule=globalfuncs.getFileHandlerModule(item.getFileHandlerName())
            logger.info('[%s]Processing %s with %s', mp.current_process().name, filename, fileHandlerModule.__name__)
            result=fileHandlerModule.processFile(filename, dataHandlersModules, logConfig)
            logger.debug('[%s]%s: Returned from %s', mp.current_process().name, filename, fileHandlerModule.__name__)
            with totals['totalFilesScanned'].get_lock():
                totals['totalFilesScanned'].value+=1
            with totals['totalBytes'].get_lock():
                totals['totalBytes'].value+=item.getFileSize()
            
            if len(result['matches'])>0:
                logger.debug('%s: %s matches found', filename, str(result['matches'].keys()))
                totals['totalResults'].value+=globalfuncs.countResults(result['matches'])
                for q in [qName for qName in queues.keys() if qName.endswith('_resultsQ')]:
                    queues[q].put(result)
    except KeyboardInterrupt:
        for q in [qName for qName in queues.keys() if qName.endswith('_resultsQ')]:
            globalfuncs.clearQ(queues[q])
    finally:
        logger.info('[%s]FileWorkerProcess terminated', mp.current_process().name)
        with activeFilesQProcesses.get_lock():
            activeFilesQProcesses.value-=1
            logger.info('FileWorkerProcesses remaining: %d', activeFilesQProcesses.value)
        # To allow the queue to shutdown properly, remove the last item from the filesQ if we're the last filesQ processor still running
        if activeFilesQProcesses.value==0:
            logger.debug('[%s]Last FileWorkerProcess terminated.  Clearing filesQ.', mp.current_process().name)
            queues['filesQ'].get()
        del logger


def resultsDispatcher(config: classes.Config, 
                      queues: dict,
                      stopEvent: mp.Event,
                     ) -> list:

    '''Setup the results queue handlers for the enabled output types.
    
    Returns a list of mp.Process() objects which will be started later'''
    
    try:
        logger=logging.getLogger('resultsDispatcher')
        logger.addHandler(QueueHandler(queues['logQ']))
        logger.propagate=False
        
        procs=list()

        for resultsType in config.getEnabledOutputTypes():
            try:
                makedirs(str(Path(config.getOutputFile(resultsType)).absolute().parent), exist_ok=True)
            except Exception as e:
                console.error(str(e))
                stopEvent.set()
            module=globalfuncs.getOutputHandlerModule(resultsType)
            qName=resultsType+'_resultsQ'
            proc=mp.Process(target=module.processResult, name=resultsType+'_process', args=(config.getOutputFile(resultsType), queues[qName], stopEvent), daemon=True)
            procs.append(proc)
    except KeyboardInterrupt:
        pass

    return procs


def progressLineWorker(totals: dict, 
                       logQ: mp.Queue, 
                       startTime: datetime,
                       stopEvent: mp.Event,
                      ):

    def _printProgressLine():
        INTERVAL = 1
        lastLineLen = 0

        while True:
            screenWidth=console.getTerminalSize()[0]
            line='%s | Folders scanned: %d | Files identified: %d | Files scanned: %d (%s) | Results found: %d' % (str(datetime.now() - startTime).split('.')[0], totals['totalDirs'].value, totals['totalFilesFound'].value, totals['totalFilesScanned'].value, globalfuncs.sizeof_fmt(totals['totalBytes'].value), totals['totalResults'].value)
            if len(line) > screenWidth:
                line=line[:screenWidth-1]

            console.status(line+' '*(lastLineLen-len(line)))
            lastLineLen=len(line)

            #Placing the event handler at the end will ensure that the last update is processed before terminating the thread
            if stopEvent.is_set():
                break
            sleep(INTERVAL)

    try:
        logger=logging.getLogger('progressLineWorker')
        logger.addHandler(QueueHandler(logQ))
        logger.propagate=False
        logger.info('progressLineWorker started')

        console.normal('If needed, press CTRL-C to terminate scan')
            
        if globalfuncs.getOSType()=='linux':
            _printProgressLine()
        else:
            with keep.presenting() as k:
                if k.success:
                    console.normal('Sleep prevention enabled.')
                else:
                    console.warn('Sleep prevention was unsuccessful.  System may go to sleep during scan.')
                _printProgressLine()
    except KeyboardInterrupt:
        pass
    finally:
        logger.info('progressLineWorker stopped')

def main():
    start=datetime.now()
    args=commandLineParser()
    CLEANED=False

    if args.cpuCount:
        print('CPU cores:', cpu_count())
        sys.exit(globalfuncs.errorCodes['ok'])

    if args.listDH:
        print('Data handler modules: ', globalfuncs.getSupportedDataHandlerNames())
        sys.exit(globalfuncs.errorCodes['ok'])

    if args.listFT:
        print('File extns: ', globalfuncs.getSupportedFileExts())
        print('MIME types: ', globalfuncs.getSupportedFileMimes())
        sys.exit(globalfuncs.errorCodes['ok'])

    if len(args.createConfigFile) >0:
        tomlFile = str(args.createConfigFile) if str(args.createConfigFile).endswith('.toml') else str(args.createConfigFile)+'.toml'
        configFileWritten=globalfuncs.writeDefaultConfig(tomlFile)

        if configFileWritten=='Success':
            console.normal('Default configuration written to '+args.createConfigFile)
            sys.exit(globalfuncs.errorCodes['ok'])
        else:
            console.error('Config file not written: %s' % (configFileWritten))
            sys.exit(globalfuncs.errorCodes['unknown'])

    if args.defaultConfig:
        config=classes.Config(configFile='', useDefault=True)
    else:
        config=classes.Config(configFile=args.configFile)

    if args.maxProc>0:
        config.setMaxProcs(min(cpu_count(), args.maxProc))

    # Create queues and other structures needed for asynchronous implementation
    totals={k: mp.Value(c_uint64, 0) for k in ['totalDirs', 'totalFilesFound', 'totalFilesScanned', 'totalBytes', 'totalResults']}
    queues={name: mp.Queue() for name in ['logQ', 'dirsQ', 'filesQ', 'totalsQ',]}
    activeFilesQProcesses=mp.Value(c_int, 0)
    processes = dict()
    for resultsType in config.getEnabledOutputTypes():
        name=resultsType+'_resultsQ'
        queues.update({name: mp.Queue()})
    stopEvent=mp.Event()
    stopEvent.clear()

    # Configure logging
    makedirs(str(Path(config.getLogFile()).absolute().parent), exist_ok=True)
    processes['logger'] = [mp.Process(target=logProcessor, args=(config, queues['logQ'], stopEvent), name='logger_process', daemon=True)]
    processes['logger'][0].start()
    logger=logging.getLogger()
    logger.setLevel(config.getLogLevel())
    logger.addHandler(QueueHandler(queues['logQ']))
    logger=logging.getLogger('main')
        
    logger.info('Command line arguments: %s', sys.argv[1:])
    logger.info("Configuration: %s", json.dumps(config.getConfig()))

    if len(config.getMimeTypes()) == 0:        
        logger.info("MIME detection disabled.")

    if not globalfuncs.isAdmin():
        message='Not running as an administrator. File system access may be restricted.'
        console.warn(message)
        logger.info(message)

    if globalfuncs.getOSType() == 'linux':
        console.warn('Sleep prevention disabled on Linux. Consider using \'screen\' or \'tmux\' to ensure that PIIDigger survives an SSH disconnect.')
    
    console.normal('Scanning %s for files matching %s' % (config.getStartDirs(), config.getDataHandlers()))
    
    #####################################
    # Bring up the pipe line in reverse order -- starting with results and working back to scanning for directories
    #####################################
        
    # Setup each of the subprocesses that are needed
    processes['progressLine']=[mp.Process(target=progressLineWorker, args=(totals,queues['logQ'], start, stopEvent), name='progressLine_process', daemon=True)]
    processes['outputHandlers']=resultsDispatcher(config, queues, stopEvent)
    processes['findFiles']=[mp.Process(target=filescan.findFilesWorker, args=(config, queues, totals, stopEvent,), name='findFiles_process', daemon=True)]
    processes['filesQWorkers']=[mp.Process(target=fileHandlerDispatcher, args=(config, queues, totals, stopEvent, activeFilesQProcesses), name='fileHandler'+str(i)+'_process', daemon=True) for i in range(config.getMaxProcs())]
    processes['findDirs']=[mp.Process(target=filescan.findDirsWorker, args=(config, queues, totals, stopEvent,), name='findDirs_process', daemon=True)]
    console.normal('Starting %d file scanner processes' % (config.getMaxProcs()))

    # Start each process in the processes dictionary
    for processType in processes:
        for process in processes[processType]:
            try:
                if not process.is_alive():
                    process.start()
                    logger.info('Starting process %s (%s)', process.name, process.pid)
            except Exception as e:
                logger.info('Error starting %s: %s', process.name, str(e))


    ##################################
    # Wait for the threads/processes to complete by joining each of the threads/processes in the pipeline in pipeline order
    ##################################

    try:
        for processType in ['findDirs', 'findFiles', 'filesQWorkers']:
            for process in processes[processType]:
                logger.info('Joining process %s (%s)', process.name, process.pid)
                process.join()
                logger.info('Returned from process: %s (%s)', process.name, process.pid)
    except KeyboardInterrupt:
        #console.normal('\nUser terminated scan')
        cleanup(processes, queues, stopEvent, abort=True)
        CLEANED=True
    except Exception:
        console.error('An unknown error was encountered.  Error message was captured in %s.' % config.getLogFile())
        logger.error(traceback.print_exc())
    else:
        # Send the sentinel to the results queues and then block until they shutdown properly.
        for q in [qName for qName in queues.keys() if qName.endswith('_resultsQ')]:
            queues[q].put(None)        
        for process in processes['outputHandlers']:
            process.join()
    finally:
        # If the logger hasn't already been shutdown by a KeyboardInterrupt or other event
        if processes['logger'][0].is_alive():
            logger.info('Scanned %d files for %s', totals['totalFilesScanned'].value, globalfuncs.sizeof_fmt(totals['totalBytes'].value))
            logger.info('Found %d files with matching content', totals['totalResults'].value)

            # Terminate the logProcessor and progress line worker
            logger.info('Scan complete.  Shutting down.')
            queues['logQ'].put(None)
        
        stopEvent.set()        
        processes['progressLine'][0].join()
        console.normal('\nWaiting for logger process to terminate.')
        
        # Make sure all of the queues are empty so that the MPQueueing support threads will shutdown
        if not CLEANED: 
            cleanup(processes, queues, stopEvent)

        console.normal('Scan complete.')
            
if __name__ == '__main__':
    main()


import logging
import sys

from chardet import UniversalDetector

moduleName='getencoding'

logger=logging.getLogger(moduleName)

def getEncoding(filename: str,) -> str:
    '''
    Uses chardet to indenty the file encoding by reading MAXLINES of data.

    '''

# Determing the encoding is not an exact science.  In this method, we read up to MAXLINES of data for each encoding type and if no error is thrown, then that's our guess.
# If we do throw a UnicodeDecodeError exception, then we try the next one.

# Encodings should be listed in order of preference.  For instance, try UTF-8 before trying the more generic ASCII.  
# Taken from https://docs.python.org/3.9/library/codecs.html#standard-encodings and includes the ones that support Western and many European languages
    logger=logging.getLogger(moduleName)
    detector = UniversalDetector()
    detector.logger.level=logging.INFO
        
    try:
        with open(filename, 'rb') as f:
            for line in f.readlines():
                detector.feed(line)
                if detector.done: 
                    break
        detector.close()
        guess=detector.result['encoding']
    except Exception as e:
        logger.info('%s: %s', filename, str(e))
        # Mimic the chardet.detector output to preserve code integrety for function consumers
        guess=None
    
    logger.debug('Filename %s chardet results: %s', filename, str(guess))

    return guess


if __name__ == "__main__":

    args=sys.argv[1:]
    DEBUG=True    
    for arg in args:
        try:
            print('Filename: %s\nEncoding: %s\n' % (arg, getEncoding(arg)))
        except FileNotFoundError:
            print('Filename not found: %s\n' % arg)



import logging
import openpyxl
import warnings
from logging.handlers import QueueHandler

from openpyxl.utils.exceptions import *
from zipfile import BadZipFile

# Only need to modify the path during unit testing of this file.
if __name__=='__main__':
    import sys
    from pathlib import Path
    sys.path.insert(0, str(Path(__file__).absolute().parent.parent / ''))

import piidigger.globalfuncs as globalfuncs

# Ignore the UserWarning message from OpenPyXL that seem to pop up here and there
warnings.filterwarnings('ignore', category=UserWarning, module='openpyxl')

# Each filehandler must have the following:
#   "handles" -     dictionary to identify lists of file extensions and mime types that the handler will manage.
#                   This will be read by globals upon initial load to build the full list of supported mime types and file extensions
#   "processFile" - Function that manages opening and reading of the file.  The main module will call this handler wtih the "processFile(filename)" function.
#                   processFile should provide the lines of text to each of the dataHandlers

handles={
    'ext': [
        '.xlsx',
        '.xlsm',
        '.xlst',
        '.xltm',
    ],
    'mime': [
        'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        'application/vnd.ms-excel.sheet.macroEnabled',
        'application/vnd.ms-excel.template',
        ],
}

def processFile(filename: str, dataHandlers: list, logConfig: dict) -> list:
    ''''
    Handle all file IO and text extraction operations for this file type.  Returns a list of results that have been validated by each datahandler.  
    "filename" is a string of the path and filename to process.  "handlers" is passed as a list of module objects that are called directly by processFile.
    '''

    logger=logging.getLogger('xlsx-handler')
    if not logger.handlers:
        logger.addHandler(QueueHandler(logConfig['q']))
    logger.setLevel(logConfig['level'])
    logger.propagate=False
    
    results={
        'filename': filename,
        'matches': dict()
    }

    try:
        # Don't use "on_demand" in order to keep the code simpler.  All worksheets are loaded into RAM.
        # Some spreadsheet dimensions can't be accurately determined -- e.g. if there's a lot of extraneous formatting to make it look "pretty"
        # We build a safety valve so that it stops after the first 5000 rows and 5000 columns.  If the "interesting data" is present outside of these limits...
        # well... it's probably not the ONLY instance of such data.

        book=openpyxl.load_workbook(filename=filename, read_only=True, data_only=True,)
        logger.debug('%s: Read %d worksheets', filename, len(book.sheetnames))
        for sheet in book.sheetnames:
            logger.debug('%s: Processing worksheet: %s', filename, str(sheet))
            activeSheet=book[sheet]
            content=str()
            blankRowCount=0
            rowCount=0
            # create a string with all of the content of this sheet
            # Iterate through each cell in each row.  If we reach a limit of blank cells, move to the next row
            # If we reach a limit of blank rows, then move to the next sheet.
            for row in activeSheet.iter_rows(values_only=True):
                rowCount+=1
                rowHasData=False
                blankColCount=0
                for item in row:
                    if isinstance(item, openpyxl.cell.cell.MergedCell):
                        continue
                    if item is None or item == '':
                        blankColCount+=1
                        if blankColCount>globalfuncs.excelBlankColLimit:
                            break
                        continue
                    content += str(item) + ' '
                    rowHasData=True
                if rowHasData:
                    blankRowCount=0
                    content += ' '
                else:
                    blankRowCount+=1
                    if blankRowCount>globalfuncs.excelBlankRowLimit:
                        logger.debug('%s[Sheet %s]: Blank row count exceeded at row %d', filename, sheet, rowCount)
                        break

            logger.debug('%s[Sheet %s]: Read content (%d bytes)', filename, sheet, len(content))
            chunks = globalfuncs.makeChunks(content)
            logger.debug('%s[Sheet %s]: Created %d chunks', filename, sheet, len(chunks))

            del content

            for handler in dataHandlers:
                logger.debug('%s[Sheet %s]: Processing %d chunks with %s', filename, sheet, len(chunks), handler.dhName)
                for chunk in chunks:
                    results=globalfuncs.processMatches(results, handler.findMatch(chunk), handler.dhName)
        book.close()
    except FileNotFoundError:
        logger.error('Previously discovered file no longer exists: %s. File skipped', filename)
    except PermissionError as e:
        logger.error('PermissionError adding %s.  File skipped.  Error message: %s', filename, str(e))
    except OSError as e:
        logger.error('OSError adding %s.  File skipped.  Error message: %s', filename, str(e))
    except CellCoordinatesException as e: 
        logger.error('%s: %s', filename, e)
    except IllegalCharacterError as e:
        logger.error('%s: %s', filename, e)
    except InvalidFileException as e:
        logger.error('%s: %s', filename, e)
    except SheetTitleException as e:
        logger.error('%s: %s', filename, e)
    except UserWarning as e:
        logger.error('%s: %s', filename, e)
    except BadZipFile as e:
        logger.error('%s: %s', filename, e)
    except Exception as e:
        logger.error('Unknown exception on file %s.  File skipped.  Error message: %s', filename, str(e))


    
    # Since Python sets aren't serializable as a JSON object type, we'll convert our results to Lists now.
    logger.debug('%s: Rebuilding result sets into lists', filename)
    for handler in results['matches']:
        for key in results['matches'][handler]:
            l=list(results['matches'][handler][key])
            results['matches'][handler][key]=l

    logger.debug('%s: Processing complete', filename)
    return results


def main():
    from sys import argv
    from multiprocessing import Queue
    handlers=globalfuncs.getAllDataHandlerModules()
    logConfig={'q': Queue(),
                'level': "DEBUG",
                }
    for arg in argv[1:]:

        print(processFile(arg, handlers, logConfig))

if __name__ == '__main__':
    main()




# This looks like a better cross-platform implementation.  Will need to look at implementing as the current "magic" package fails on Windows
# https://github.com/cdgriffith/puremagic

moduleName='getmime'

import os, sys

try:
    import puremagic 
except Exception:
    pass

def testMagic() -> bool:
    return 'puremagic' in sys.modules

def getMime(filename: str) -> str:
    if not isinstance(filename, str):
        filename=str(filename)
    if os.path.isdir(filename):
        return "Directory"
    
    try:
        mimeType = puremagic.from_file(filename, mime=True) if testMagic() else None
    except Exception: 
        mimeType = None
        
    
    return mimeType

if __name__ == "__main__":
    libmagicLoaded = testMagic()
        
    for arg in sys.argv[1:]:
        print('Filename: %s\nMime: %s\n' % (arg, getMime(arg)))
